//+------------------------------------------------------------------+
//|                                              ShadowTwoLayers.mq5 |
//|                              Copyright  2015, Vladimir Karputov |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2015, Vladimir Karputov"
#property link      "http://wmua.ru/slesar/"
#property version   "1.00"
#property description "Shadows on the two layers."
#property script_show_inputs
#include <Canvas\Canvas.mqh>
#include <Math\Alglib\alglib.mqh>
//--- input
input uint  radius=4;               // blur radius
input color clrShadow=clrBlack;     // shadow color
input uchar ShadowTransparence=160; // shadows transparency
input int   ShadowShift=3;          // shadow shift
input color clrDraw=clrBlue;        // shadow color
input uchar DrawwTransparence=255;  // transparency draws
//---
int         NNodes;
//--- variable width and height of the chart
int   ChartWidth=-1;
int   ChartHeight=-1;
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//--- variable width and height of the chart
   ChartWidth=ChartWidthInPixels(0);
   ChartHeight=ChartHeightInPixelsGet(0,0);
//--- create canvas
   CCanvas CanvasShadow;
   CCanvas CanvasDraw;
   if(!CanvasShadow.CreateBitmapLabel("ShadowLayer",0,0,ChartWidth,
      ChartHeight,COLOR_FORMAT_ARGB_NORMALIZE))
     {
      Print("Error creating canvas: ",GetLastError());
      return;
     }
   if(!CanvasDraw.CreateBitmapLabel("DrawLayer",0,0,ChartWidth
      ,ChartHeight,COLOR_FORMAT_ARGB_NORMALIZE))
     {
      Print("Error creating canvas: ",GetLastError());
      return;
     }
//--- draw on canvas
   CanvasShadow.Erase(ColorToARGB(clrNONE,0));
   CanvasShadow.FillRectangle(ChartWidth/10,ChartHeight/10,
                              ChartWidth/2-ChartWidth/10,ChartHeight/10*9,ColorToARGB(clrShadow,ShadowTransparence));
   CanvasShadow.FillRectangle(ChartWidth/2,ChartHeight/12,ChartWidth/3*2,
                              ChartHeight/2,ColorToARGB(clrShadow,ShadowTransparence));
   CanvasShadow.Update();
//---
   CanvasDraw.Erase(ColorToARGB(clrNONE,0));
   CanvasDraw.FillRectangle(ChartWidth/10-ShadowShift,ChartHeight/10-ShadowShift,ChartWidth/2-ChartWidth/10-ShadowShift,
                            ChartHeight/10*9-ShadowShift,ColorToARGB(clrDraw,DrawwTransparence));
   CanvasDraw.Update();
//+------------------------------------------------------------------+
//| Blur image using a Gaussian filter                               |
//+------------------------------------------------------------------+
//--- Remember the initial value
   uint start=GetTickCount();
//---
   NNodes=(int)radius*2+1;
//---
   uint res_data[];              // array for receiving data from the graphical resource
   uint res_width=ChartWidth;    // width of the resource
   uint res_height=ChartHeight;  // height of the resource
//+------------------------------------------------------------------+
//| reads data from the graphical resource                           |
//+------------------------------------------------------------------+
   ResetLastError();
   if(!ResourceReadImage(CanvasShadow.ResourceName(),res_data,res_width,res_height))
     {
      Print("Error reading data from the graphical resource ",GetLastError());
      Print("attempt number two");
      //--- attempt number two: now the picture width and height are known
      ResetLastError();
      if(!ResourceReadImage(CanvasShadow.ResourceName(),res_data,res_width,res_height))
        {
         Print("Error reading data from the graphical resource ",GetLastError());
         return;
        }
     }
//---
   if(radius>=res_width/2 || radius>=res_height/2)
     {
      Print("Error! Image size is very small or very large blur");
      return;
     }
//+------------------------------------------------------------------+
//| decomposition of pictures on the components r, g, b              |
//+------------------------------------------------------------------+
   int      size=ArraySize(res_data);
   uint     res_blur_hor[];               // temp array for receiving data from the graphical resource
   if(ArrayResize(res_blur_hor,size)==-1)
     {
      Print("Error setting the new size of an array of \"res_blur_hor\"");
      return;
     }
//---
   uint     clr_temp;                     // color in ARGB format
   uchar    a_data[],r_data[],g_data[],b_data[];      // arrays of colors R, G, B
   uchar    a1_data[],r1_data[],g1_data[],b1_data[];  // arrays of colors R, G, B
//---
   if(ArrayResize(a_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"a_data\"");
      return;
     }
//---
   if(ArrayResize(r_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"r_data\"");
      return;
     }
//---
   if(ArrayResize(g_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"g_data\"");
      return;
     }
//---
   if(ArrayResize(b_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"b_data\"");
      return;
     }
//---
   if(ArrayResize(a1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"a1_data\"");
      return;
     }
//---
   if(ArrayResize(r1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"r1_data\"");
      return;
     }
//---
   if(ArrayResize(g1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"g1_data\"");
      return;
     }
//---
   if(ArrayResize(b1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"b1_data\"");
      return;
     }
//---
   double weights[];                      // array of weights
//---
   if(!GetQuadratureWeights(1,NNodes,weights))
      return;
//---
   for(int i=0;i<size;i++)
     {
      clr_temp=res_data[i];
      a_data[i]=GETRGBA(clr_temp);
      r_data[i]=GETRGBR(clr_temp);
      g_data[i]=GETRGBG(clr_temp);
      b_data[i]=GETRGBB(clr_temp);
     }
//+------------------------------------------------------------------+
//| blur horizontal (axis X)                                         |
//+------------------------------------------------------------------+
   uint XY;             // pixel coordinate in the array
   double   a_temp=0.0,r_temp=0.0,g_temp=0.0,b_temp=0.0;
   int      coef=0;
   int      j=(int)radius;
   for(uint Y=0;Y<res_height;Y++) // cycle on image width
     {
      for(uint X=radius;X<res_width-radius;X++) // cycle on image height
        {
         XY=Y*res_width+X;
         a_temp=0.0; r_temp=0.0; g_temp=0.0; b_temp=0.0;
         coef=0;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            a_temp+=a_data[XY+i]*weights[coef];
            r_temp+=r_data[XY+i]*weights[coef];
            g_temp+=g_data[XY+i]*weights[coef];
            b_temp+=b_data[XY+i]*weights[coef];
            coef++;
           }
         a_data[XY]=(uchar)MathRound(a_temp);
         r_data[XY]=(uchar)MathRound(r_temp);
         g_data[XY]=(uchar)MathRound(g_temp);
         b_data[XY]=(uchar)MathRound(b_temp);
        }
      //--- remove artifacts on the left
      for(uint x=0;x<radius;x++)
        {
         XY=Y*res_width+x;
         a_data[XY]=a_data[Y*res_width+radius];
         r_data[XY]=r_data[Y*res_width+radius];
         g_data[XY]=g_data[Y*res_width+radius];
         b_data[XY]=b_data[Y*res_width+radius];
        }
      //--- remove artifacts on the right
      for(uint x=res_width-radius;x<res_width;x++)
        {
         XY=Y*res_width+x;
         a_data[XY]=a_data[(Y+1)*res_width-radius-1];
         r_data[XY]=r_data[(Y+1)*res_width-radius-1];
         g_data[XY]=g_data[(Y+1)*res_width-radius-1];
         b_data[XY]=b_data[(Y+1)*res_width-radius-1];
        }
     }
//+------------------------------------------------------------------+
//| blur vertically (axis Y)                                         |
//+------------------------------------------------------------------+
   int dxdy=0;
   for(uint X=0;X<res_width;X++) // cycle on image height
     {
      for(uint Y=radius;Y<res_height-radius;Y++) // cycle on image width
        {
         XY=Y*res_width+X;
         a_temp=0.0; r_temp=0.0; g_temp=0.0; b_temp=0.0;
         coef=0;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            dxdy=i*(int)res_width;
            a_temp+=a_data[XY+dxdy]*weights[coef];
            r_temp+=r_data[XY+dxdy]*weights[coef];
            g_temp+=g_data[XY+dxdy]*weights[coef];
            b_temp+=b_data[XY+dxdy]*weights[coef];
            coef++;
           }
         a1_data[XY]=(uchar)MathRound(a_temp);
         r1_data[XY]=(uchar)MathRound(r_temp);
         g1_data[XY]=(uchar)MathRound(g_temp);
         b1_data[XY]=(uchar)MathRound(b_temp);
        }
      //--- remove artifacts on the top
      for(uint y=0;y<radius;y++)
        {
         XY=y*res_width+X;
         a1_data[XY]=a1_data[X+radius*res_width];
         r1_data[XY]=r1_data[X+radius*res_width];
         g1_data[XY]=g1_data[X+radius*res_width];
         b1_data[XY]=b1_data[X+radius*res_width];
        }
      //--- remove artifacts on the down
      for(uint y=res_height-radius;y<res_height;y++)
        {
         XY=y*res_width+X;
         a1_data[XY]=a1_data[X+(res_height-1-radius)*res_width];
         r1_data[XY]=r1_data[X+(res_height-1-radius)*res_width];
         g1_data[XY]=g1_data[X+(res_height-1-radius)*res_width];
         b1_data[XY]=b1_data[X+(res_height-1-radius)*res_width];
        }
     }
//---
   for(int i=0;i<size;i++)
     {
      clr_temp=ARGB(a1_data[i],r1_data[i],g1_data[i],b1_data[i]);
      res_data[i]=clr_temp;
     }
   for(uint X=0;X<res_width;X++)
     {
      for(uint Y=radius;Y<res_height-radius;Y++)
        {
         XY=Y*res_width+X;
         CanvasShadow.PixelSet(X,Y,res_data[XY]);
        }
     }
   CanvasShadow.Update();
   CanvasDraw.Update();
   Sleep(21000);
//---
   ResourceFree("res");
   ResourceFree("res_a");
//--- Get the spent time in milliseconds
   uint time=GetTickCount()-start;
//--- Output a message to the Experts journal
//PrintFormat("Blur image using a Gaussian filter took %d ms",time);
//---
   CanvasShadow.Destroy();
   CanvasDraw.Destroy();
   return;
  }
//+------------------------------------------------------------------+
//| Chart property width                                             |
//+------------------------------------------------------------------+
int ChartWidthInPixels(const long chart_ID=0)
  {
//--- prepare the variable to get the property value
   long result=-1;
//--- reset the error value
   ResetLastError();
//--- receive the property value
   if(!ChartGetInteger(chart_ID,CHART_WIDTH_IN_PIXELS,0,result))
     {
      //--- display the error message in Experts journal
      Print(__FUNCTION__+", Error Code = ",GetLastError());
     }
//--- return the value of the chart property
   return((int)result);
  }
//+------------------------------------------------------------------+
//| Chart property height                                            |
//+------------------------------------------------------------------+
int ChartHeightInPixelsGet(const long chart_ID=0,const int sub_window=0)
  {
//--- prepare the variable to get the property value
   long result=-1;
//--- reset the error value
   ResetLastError();
//--- receive the property value
   if(!ChartGetInteger(chart_ID,CHART_HEIGHT_IN_PIXELS,sub_window,result))
     {
      //--- display the error message in Experts journal
      Print(__FUNCTION__+", Error Code = ",GetLastError());
     }
//--- return the value of the chart property
   return((int)result);
  }
//+------------------------------------------------------------------+
//| Gets array of quadrature weights                                 |
//+------------------------------------------------------------------+
bool GetQuadratureWeights(const double mu0,const int n,double &w[])
  {
   CAlglib alglib;            // static member of class CAlglib
   double      alp[];         // array alpha coefficients 
   double      bet[];         // array beta coefficients 
   ArrayResize(alp,n);
   ArrayResize(bet,n);
   ArrayInitialize(alp,1.0);  // initializes a numeric array alpha
   ArrayInitialize(bet,1.0);  // initializes a numeric array beta
//---
   double      out_x[];
   int         inf=0;
//| Info    -   error code:                                          |
//|                 * -3    internal eigenproblem solver hasn't      |
//|                         converged                                |
//|                 * -2    Beta[i]<=0                               |
//|                 * -1    incorrect N was passed                   |
//|                 *  1    OK                                       |
   alglib.GQGenerateRec(alp,bet,mu0,n,inf,out_x,w);
   if(inf!=1)
     {
      Print("Call error in CGaussQ::GQGenerateRec");
      return(false);
     }
   return(true);
  }
//+------------------------------------------------------------------+
